from pathlib import Path
from yaml import load, Loader

class Config:
    def __init__(self, path: Path = None) -> None:
        if path is None:
            self.config_path = Path.home().joinpath(".stakenix").joinpath("config.yaml")
        else:
            self.config_path = path
    
    def get_config(self) -> dict:
        with open(self.config_path, 'r') as file:
            return load(file, Loader=Loader)


from os import getenv
from pathlib import Path
from sys import platform
from yaml import safe_dump


def generate_env_config() -> None:
    config = {
        "sql": {
            "mysql": {
                "driver": {"name": "mysql+mysqlconnector", "connection_args": {}},
                "databases": {
                    "default": {
                        "host": getenv("MYSQL_DEFAULT_HOST"),
                        "port": getenv("MYSQL_DEFAULT_PORT"),
                        "username": getenv("MYSQL_DEFAULT_USERNAME"),
                        "password": getenv("MYSQL_DEFAULT_PASSWORD"),
                    },
                    "lucky_partners": {
                        "host": getenv("MYSQL_LP_HOST"),
                        "port": getenv("MYSQL_LP_PORT"),
                        "username": getenv("MYSQL_LP_USERNAME"),
                        "password": getenv("MYSQL_LP_PASSWORD"),
                    },
                     "lucky_partners_backup": {
                        "host": getenv("MYSQL_LP_BACKUP_HOST"),
                        "port": getenv("MYSQL_LP_BACKUP_PORT"),
                        "username": getenv("MYSQL_LP_BACKUP_USERNAME"),
                        "password": getenv("MYSQL_LP_BACKUP_PASSWORD"),
                    },
                },
            },
            "postgresql": {
                "driver": {
                    "name": "postgresql",
                    "connection_args": {"options": "-csearch_path={}"},
                },
                "databases": {
                    "default": {
                        "host": getenv("PG_DEFAULT_HOST"),
                        "port": getenv("PG_DEFAULT_PORT"),
                        "username": getenv("PG_DEFAULT_USERNAME"),
                        "password": getenv("PG_DEFAULT_PASSWORD"),
                        "database_name": "analyst_db",
                    },
                    "alpa": {
                        "host": getenv("PG_ALPA_HOST"),
                        "port": getenv("PG_ALPA_PORT"),
                        "username": getenv("PG_ALPA_USERNAME"),
                        "password": getenv("PG_ALPA_PASSWORD"),
                        "database_name": "rocketplay",
                    },
                    "tableau": {
                        "host": getenv("PG_TABLEAU_HOST"),
                        "port": getenv("PG_TABLEAU_PORT"),
                        "username": getenv("PG_TABLEAU_USERNAME"),
                        "password": getenv("PG_TABLEAU_PASSWORD"),
                        "database_name": "workgroup",
                    }
                },
            },
            "clickhouse": {
                "driver": {"name": "clickhouse", "connection_args": {}},
                "databases": {
                    "default": {
                        "host": getenv("CH_DEFAULT_HOST"),
                        "port": getenv("CH_DEFAULT_PORT"),
                        "username": getenv("CH_DEFAULT_USERNAME"),
                        "password": getenv("CH_DEFAULT_PASSWORD"),
                    }
                },
            },
            "mssql": {
                "driver": {
                    "name": "mssql+pymssql" if platform == "linux" else "mssql",
                    "connection_args": {}
                    if platform == "linux"
                    else {"driver": "ODBC Driver 17 for SQL Server"},
                },
                "databases": {
                    "default": {
                        "host": getenv("MSSQL_DEFAULT_HOST"),
                        "port": getenv("MSSQL_DEFAULT_PORT"),
                        "username": getenv("MSSQL_DEFAULT_USERNAME"),
                        "password": getenv("MSSQL_DEFAULT_PASSWORD"),
                        "database_name": "BPMonline7102CustomerCenterSoftkeyRUS_1453921_0613",
                    }
                },
            },
        },
        "nosql": {
            "mongodb": {
                "databases": {
                    "pobeda": {
                        "database_name": "pobeda",
                        "host": getenv("POBEDA_MONGODB_HOST"),
                        "username": getenv("POBEDA_MONGODB_USERNAME"),
                        "password": getenv("POBEDA_MONGODB_PASSWORD"),
                        "ssh_host": getenv("POBEDA_SSH_HOST"),
                        "ssh_username": getenv("POBEDA_SSH_USERNAME"),
                    },
                    "lara": {
                        "database_name": "lotoru",
                        "host": getenv("LARA_MONGODB_HOST"),
                        "username": getenv("LARA_MONGODB_USERNAME"),
                        "password": getenv("LARA_MONGODB_PASSWORD"),
                        "ssh_host": getenv("LARA_SSH_HOST"),
                        "ssh_username": getenv("LARA_SSH_USERNAME"),
                    },
                    "vipt": {
                        "database_name": "vipt",
                        "host": getenv("VIPT_MONGODB_HOST"),
                        "username": getenv("VIPT_MONGODB_USERNAME"),
                        "password": getenv("VIPT_MONGODB_PASSWORD"),
                        "ssh_host": getenv("VIPT_SSH_HOST"),
                        "ssh_username": getenv("VIPT_SSH_USERNAME"),
                    },
                    "mk5": {
                        "database_name": "vulkan",
                        "host": getenv("MK5_MONGODB_HOST"),
                        "username": getenv("MK5_MONGODB_USERNAME"),
                        "password": getenv("MK5_MONGODB_PASSWORD"),
                        "ssh_host": getenv("MK5_SSH_HOST"),
                        "ssh_username": getenv("MK5_SSH_USERNAME"),
                    },
                    "vipclub": {
                        "database_name": "vipclub",
                        "host": getenv("VIPCLUB_MONGODB_HOST"),
                        "username": getenv("VIPCLUB_MONGODB_USERNAME"),
                        "password": getenv("VIPCLUB_MONGODB_PASSWORD"),
                        "ssh_host": getenv("VIPCLUB_SSH_HOST"),
                        "ssh_username": getenv("VIPCLUB_SSH_USERNAME"),
                    },
                }
            }
        },
        "grafana": {
            "url": getenv("GRAFANA_URL"),
            "headers": {
                "Authorization": getenv("GRAFANA_AUTH"),
                "Content-Type": getenv("GRAFANA_CONTENT_TYPE"),
                "Accept": getenv("GRAFANA_ACCEPT"),
            },
        },
    }

    path = Path.home().joinpath(".stakenix")
    config_path = path.joinpath("config.yaml")

    if not path.exists():
        path.mkdir()

    with open(config_path, "w") as file:
        safe_dump(config, file, sort_keys=False)

    print("Config successfully generated")
    print(f"Path to config: {config_path}")


if __name__ == "__main__":
    generate_env_config()


from collections.abc import Iterable
from functools import reduce
from typing import List, Any, Union, Iterable

from bson.objectid import ObjectId
from pandas import DataFrame, concat, Series, merge


def bson_to_string(df: DataFrame) -> DataFrame:
    for column in df.columns:
        if not df[column].dropna().empty and isinstance(
                df.at[df[column].first_valid_index(), column], ObjectId):
            df[column] = df[column].astype("str")
    return df


def flatten(df: DataFrame) -> DataFrame:
    def _get_nested_columns(frame: DataFrame):
        columns = []
        for col in frame.columns:
            if not frame[col].dropna().empty and isinstance(
                    frame.at[frame[col].first_valid_index(), col], dict):
                columns.append(col)
        return columns

    def unpack(df: DataFrame, columns: list) -> DataFrame:
        storage = []
        for column in columns:
            df[column] = df[column].fillna({})
            placeholder = lambda val:{True: "{}{}", False: "{}_{}"}.get(val)
            tmp = df[column].apply(Series, dtype="object")

            if 0 in tmp.columns:
                tmp = tmp.drop(0, axis=1)

            storage.append(
                tmp.rename(
                    columns={c: placeholder(c.startswith("_")).format(
                        column, c) for c in tmp.columns}
                )
            )
        return concat([df.drop(columns, axis=1)]+storage, axis=1)

    nested_columns = _get_nested_columns(df.reset_index(drop=True))
    result = unpack(df.reset_index(drop=True), nested_columns)

    return result


def chunkize(data: List[Any], size: int = 5000) -> Iterable:
    if size <= 0:
        raise ValueError("param size must be greater than 0")    
    return (
        data[position:position+size] 
        for position  in range(0, len(data), size)
    )


def merger(*args) -> DataFrame:
    return reduce(lambda left, right: [merge(left[0], right[0], **right[1])], args)[0]


def str_to_bson(data: Union[str, List[str]]) -> List[ObjectId]:
    if isinstance(data, str):
        return ObjectId(data)
    elif isinstance(data, Iterable):
        if len(data) == 0:
            raise ValueError("List size shouldn't equal zero.")
        return [ObjectId(value) for value in data]
    else:
        raise ValueError("value should be single string or list of strings")